#pragma once

#include <memory>
#include <string>
#include <vector>

namespace signimage {
class SignatureImagePrivate;

struct CertificateInfo {
    std::string cert_serial;
    std::string subject;
    std::string timeValidity;
};

struct Pos {
    double x = 0;
    double y = 0;
};

struct Size {
    double width = 0;
    double height = 0;
};

struct RGBAColor {
    uint8_t r = 0;
    uint8_t g = 0;
    uint8_t b = 0;
    uint8_t alpha = 0xff;
};

struct BorderRadius {
    double cornerWidth = 0;
    double cornerHeight = 0;
};

struct Resolution {
    size_t x = 0;
    size_t y = 0;
};

class SignatureImage {
  public:
    enum TextAlignment {
        UndefinedGravity,
        ForgetGravity = 0,
        NorthWestGravity = 1,
        NorthGravity = 2,
        NorthEastGravity = 3,
        WestGravity = 4,
        CenterGravity = 5,
        EastGravity = 6,
        SouthWestGravity = 7,
        SouthGravity = 8
    };

    enum ColorModel : uint8_t { RGB, RGBA };

  private:
    struct SignatureView {
        RGBAColor bgColor;
        RGBAColor textColor;
        RGBAColor borderColor;
        BorderRadius radius;
        size_t borderWidth = 0;
        double rightMargin = 0;
        Size size;
    };

    struct TextView {
        size_t titleFontSize = 0;
        size_t fontSize = 0;
        std::string fontFamily;
        Pos titlePos;
        Pos certSerialPos;
        Pos subjectPos;
        Pos timeValidityPos;
        TextAlignment titleAlignment;
        TextAlignment contentAlignment;

        TextView()
            : titleAlignment(ForgetGravity), contentAlignment(ForgetGravity) {}
    };

    struct LogoView {
        std::string imgResol; // the string looks like <width>x<height>
        bool logoPreserveRatio = true;
        Pos logoPos;
    };

  public:
    explicit SignatureImage(std::string title, CertificateInfo *certInfo);
    explicit SignatureImage(std::string title, std::string logoFile,
                            CertificateInfo *certInfo);
    explicit SignatureImage(std::string title, unsigned char *logoData,
                            size_t logoSize, CertificateInfo *certInfo);

    void setBackgroundColor(RGBAColor rgba);
    void setTextColor(RGBAColor rgba);
    void setBorderColor(RGBAColor rgba);
    void setBorderRadius(BorderRadius radius);
    void setBorderWidth(size_t width);
    void setSignatureSize(Size size);
    void setLogoScale(std::string resolution);
    void setLogoPreserveRation(bool state);
    void setLogoPos(Pos pos);
    void setTitlePos(Pos pos);
    void setTitleAlignment(TextAlignment alignment);
    void setContentAlignment(TextAlignment alignment);
    void setCertSerialPos(Pos pos);
    void setSubjectPos(Pos pos);
    void setTimeValidityPos(Pos pos);
    void setTitleFontSize(size_t fontSize);
    void setFontSize(size_t fontSize);
    void setFontFamily(std::string font);
    void setRightMargin(double margin);

    Resolution getSignatureResolution() const;
    std::vector<unsigned char> getRawData() const;

    void draw(ColorModel colorModel = RGB);

  private:
    struct TextFieldHeight {
        double titleFieldHeight;
        double certSerialFieldHeight;
        double subjectFieldHeight;
        double timeValidityFieldHeight;
    };

    Size recalcSignatureSize();

    void updateFieldPos(TextFieldHeight textFieldH);

    bool isValidSizeValue(size_t &value) const;
    std::string getRescaleLogoImage(std::string const &targetResol,
                                    std::string const &sourceResol) const;
    void centerHLogoImagePos(double &logoPosX, size_t const &logoWidth) const;
    Resolution imgSizeFromString(std::string const &str) const;
    Resolution scaleImage(Resolution imgSize, Resolution targetSize) const;

    size_t stringToImgSize(const std::string str) const;
    std::string imgSizeToString(Resolution resol) const;

    std::string m_title;
    unsigned char *m_logo;
    size_t m_logoSize;
    std::string m_logoPath;
    CertificateInfo *m_certInfo;
    Resolution m_signResolution;
    std::vector<unsigned char> m_rawData;

    std::unique_ptr<SignatureView> m_signatureView;
    std::unique_ptr<TextView> m_textView;
    std::unique_ptr<LogoView> m_logoView;
};
} // namespace signimage
